"""Content loading tools.

Tools for loading multimedia content from URLs and external sources.
"""

from ya_agent_sdk.toolsets.core.base import BaseTool
from ya_agent_sdk.toolsets.core.content.load_media_url import LoadMediaUrlTool

tools: list[type[BaseTool]] = [
    LoadMediaUrlTool,
]

__all__ = [
    "LoadMediaUrlTool",
    "tools",
]
